@@ -1,5 +1,6 @@
 import argparse
 import atexit
+import difflib
 import os
 import re
 import shlex
@@ -9,12 +10,14 @@ import tempfile
 import textwrap
 from collections import defaultdict, deque, UserDict
 from dataclasses import dataclass
+from functools import partial
 from itertools import chain
 
 from pkgcheck import reporters, scan
 from pkgcore.ebuild.atom import MalformedAtom
 from pkgcore.ebuild.atom import atom as atom_cls
-from pkgcore.ebuild.repository import UnconfiguredTree
+from pkgcore.ebuild.repository import UnconfiguredTree, tree
+from pkgcore.ebuild.repo_objs import RepoConfig
 from pkgcore.operations import observer as observer_mod
 from pkgcore.restrictions import packages
 from snakeoil.cli import arghparse
@@ -33,6 +36,7 @@ class ArgumentParser(arghparse.ArgumentParser):
 
     def parse_known_args(self, args=None, namespace=None):
         namespace.footer = OrderedSet()
+        namespace.git_add_files = []
         namespace, args = super().parse_known_args(args, namespace)
         if namespace.dry_run:
             args.append('--dry-run')
@@ -147,41 +151,30 @@ add_actions.add_argument(
     help='stage all changed/new/removed files')
 
 
-class _HistoricalRepo(UnconfiguredTree):
+class HistoricalRepo(UnconfiguredTree):
     """Repository of historical packages stored in a temporary directory."""
 
-    def __init__(self, repo):
-        self.__parent_repo = repo
-        self.__tmpdir = tempfile.TemporaryDirectory()
-        self.__created = False
-        repo_dir = self.__tmpdir.name
-
-        # set up some basic repo files so pkgcore doesn't complain
-        os.makedirs(pjoin(repo_dir, 'metadata'))
-        with open(pjoin(repo_dir, 'metadata', 'layout.conf'), 'w') as f:
-            f.write(f"masters = {' '.join(x.repo_id for x in repo.trees)}\n")
-        os.makedirs(pjoin(repo_dir, 'profiles'))
-        with open(pjoin(repo_dir, 'profiles', 'repo_name'), 'w') as f:
-            f.write(f'{repo.repo_id}-old\n')
-        super().__init__(repo_dir)
+    def __init__(self, parent_repo, tmpdir, *args, **kwargs):
+        self.__parent_repo = parent_repo
+        # cache tmpdir to delay cleanup until exit
+        self.__tmpdir = tmpdir
+        super().__init__(*args, **kwargs)
 
     def add_pkgs(self, pkgs):
         """Update the repo with a given sequence of packages."""
         self._populate(pkgs)
-        if self.__created:
-            # notify the repo object that new pkgs were added
-            for pkg in pkgs:
-                self.notify_add_package(pkg)
-        self.__created = True
+        # notify the repo object that new pkgs were added
+        for pkg in pkgs:
+            self.notify_add_package(pkg)
 
     def _populate(self, pkgs):
         """Populate the repo with a given sequence of historical packages."""
-        paths = list({pkg.key for pkg in pkgs})
+        paths = {pkg.key for pkg in pkgs}
         old_files = subprocess.Popen(
-            ['git', 'archive', 'HEAD'] + paths,
+            ['git', 'archive', 'HEAD'] + list(paths),
             stdout=subprocess.PIPE, stderr=subprocess.PIPE,
             cwd=self.__parent_repo.location)
-        if old_files.poll():
+        if old_files.poll():  # pragma: no cover
             error = old_files.stderr.read().decode().strip()
             raise Exception(f'failed populating archive repo: {error}')
         with tarfile.open(mode='r|', fileobj=old_files.stdout) as tar:
@@ -217,15 +210,30 @@ class ChangeSummary:
     @jit_attr
     def old_repo(self):
         """Create a repository of historical packages removed from git."""
-        return _HistoricalRepo(self.repo)
+        tmpdir = tempfile.TemporaryDirectory()
+        repo_dir = tmpdir.name
 
-    def generate(self):
-        """Generate summaries for the package changes."""
+        # set up some basic repo files so pkgcore doesn't complain
+        os.makedirs(pjoin(repo_dir, 'metadata'))
+        with open(pjoin(repo_dir, 'metadata', 'layout.conf'), 'w') as f:
+            f.write(f"masters = {' '.join(x.repo_id for x in self.repo.trees)}\n")
+        os.makedirs(pjoin(repo_dir, 'profiles'))
+        with open(pjoin(repo_dir, 'profiles', 'repo_name'), 'w') as f:
+            f.write(f'{self.repo.repo_id}-old\n')
+
+        repo_config = RepoConfig(repo_dir)
+        tree_cls = partial(HistoricalRepo, self.repo, tmpdir)
+        return tree(self.options.config, repo_config, tree_cls=tree_cls)
+
+    def __str__(self):
+        """Serialize git changes into commit summary strings."""
         statuses = frozenset(x.status for x in self.changes.values())
         try:
-            return self.status_funcs[statuses](self)
-        except KeyError:
+            if s := self.status_funcs[statuses](self):
+                return s
+        except KeyError:  # pragma: no cover
             pass
+        return ''
 
 
 class MetadataSummary(ChangeSummary):
@@ -241,10 +249,11 @@ class MetadataSummary(ChangeSummary):
     def modify(self):
         """Generate summaries for modify actions."""
         atom = next(iter(self.changes))
-        self.old_repo.add_pkgs([atom])
+        pkgs = self.repo.match(atom)
+        self.old_repo.add_pkgs(pkgs)
         try:
             old_pkg = self.old_repo.match(atom)[0]
-            new_pkg = self.repo.match(atom)[0]
+            new_pkg = pkgs[0]
         except IndexError:  # pragma: no cover
             # broken ebuild should be caught during manifesting or scanning
             return
@@ -258,6 +267,8 @@ class MetadataSummary(ChangeSummary):
                 return 'add myself as a maintainer'
             if git_email in old - new:
                 return 'drop myself as a maintainer'
+            if old and not new:
+                return 'drop to maintainer-needed'
             return 'update maintainers'
         elif old_pkg.stabilize_allarches != new_pkg.stabilize_allarches:
             status = 'mark' if new_pkg.stabilize_allarches else 'drop'
@@ -270,8 +281,8 @@ class MetadataSummary(ChangeSummary):
             msg = []
             for action, data in (('add', added), ('remove', removed)):
                 if data:
-                    upstreams = [x.type for x in data]
-                    msg.append(f"{action} {', '.join(upstreams)} upstream metadata")
+                    upstreams = ', '.join(sorted(x.type for x in data))
+                    msg.append(f"{action} {upstreams} upstream metadata")
             # return action-specific shorter summary if a single type exists
             if len(msg) == 1 and len(msg[0]) <= 50:
                 return msg[0]
@@ -339,20 +350,27 @@ class PkgSummary(ChangeSummary):
     @change('R')
     def rename(self):
         """Generate summaries for rename actions."""
-        if len(self.changes) == 1 and not self.revbump:
-            # handle single, non-revbump `git mv` changes
+        if len(self.changes) == 1:
+            if not self.revbump:
+                # single, non-revbump version rename
+                change = next(iter(self.changes.values()))
+                return f'add {change.atom.fullver}, drop {change.old.fullver}'
+        elif len({x.key for x in self.changes}) == 1:
             change = next(iter(self.changes.values()))
-            return f'add {change.atom.fullver}, drop {change.old.fullver}'
+            if change.atom.key != change.old.key:
+                # package rename
+                return f'rename {change.old.key}'
 
     @change('M')
     def modify(self):
         """Generate summaries for modify actions."""
         if len(self.changes) == 1:
             atom = next(iter(self.changes))
-            self.old_repo.add_pkgs([atom])
+            pkgs = self.repo.match(atom)
+            self.old_repo.add_pkgs(pkgs)
             try:
                 old_pkg = self.old_repo.match(atom)[0]
-                new_pkg = self.repo.match(atom)[0]
+                new_pkg = pkgs[0]
             except IndexError:  # pragma: no cover
                 # broken ebuild should be caught during manifesting or scanning
                 return
@@ -380,15 +398,118 @@ class PkgSummary(ChangeSummary):
                 if len(msg) <= 50:
                     return msg
                 return action
+            else:
+                # use sourced bash env diffs to determine summaries
+                old_env = old_pkg.environment.data.splitlines()
+                new_env = new_pkg.environment.data.splitlines()
+                var_drop_re = re.compile(r'^-declare .+ (?P<name>\w+)=(?P<value>.+)$')
+                var_add_re = re.compile(r'^\+declare .+ (?P<name>\w+)=(?P<value>.+)$')
+                drop, add = {}, {}
+
+                for x in difflib.unified_diff(old_env, new_env):
+                    if mo := var_drop_re.match(x):
+                        drop[mo.group('name')] = mo.group('value')
+                    elif mo := var_add_re.match(x):
+                        add[mo.group('name')] = mo.group('value')
+
+                watch_vars = {'HOMEPAGE', 'DESCRIPTION', 'LICENSE', 'SRC_URI'}
+                array_targets = {'PYTHON_COMPAT', 'LUA_COMPAT'}
+                string_targets = {'USE_RUBY'}
+                targets = array_targets | string_targets
+
+                updated_vars = drop.keys() & add.keys()
+                if updated := sorted(watch_vars & updated_vars):
+                    return f"update {', '.join(updated)}"
+                elif (target := targets & updated_vars) and len(target) == 1:
+                    target = next(iter(target))
+                    if target in array_targets:
+                        array_re = re.compile(r'\[\d+\]="(?P<val>.+?)"')
+                        py_re = lambda x: re.sub(r'^python(\d+)_(\d+)$', r'py\1.\2', x)
+                        old = {py_re(m.group('val')) for m in re.finditer(array_re, drop[target])}
+                        new = {py_re(m.group('val')) for m in re.finditer(array_re, add[target])}
+                    else:
+                        old = set(drop[target].strip('"').split())
+                        new = set(add[target].strip('"').split())
+
+                    msg = []
+                    if added := sorted(new - old):
+                        msg.append(f"enable {', '.join(added)}")
+                    if dropped := sorted(old - new):
+                        msg.append(f"disable {', '.join(dropped)}")
+                    msg = ' and '.join(msg)
+                    if len(msg) <= 50:
+                        return msg
+                    else:
+                        return f'update {target} support'
 
 
 class GitChanges(UserDict):
     """Mapping of change objects for staged git changes."""
 
-    def __init__(self, options, changes):
-        super().__init__(changes)
+    # ebuild path regex, validation is handled on instantiation
+    _ebuild_re = re.compile(r'^(?P<category>[^/]+)/[^/]+/(?P<package>[^/]+)\.ebuild$')
+    _eclass_re = re.compile(r'^eclass/(?P<name>[^/]+\.eclass)$')
+
+    def __init__(self, options):
         self._options = options
         self._repo = options.repo
+        super().__init__(self._generate_mapping())
+
+    def _generate_mapping(self):
+        """Generate mapping for staged changes."""
+        # stage changes as requested
+        if self._options.git_add_arg:
+            git.run('add', self._options.git_add_arg, self._options.cwd)
+
+        # determine staged changes forcing rename search
+        p = git.run(
+            'diff-index', '--diff-filter=ARMD', '--find-renames',
+            '--name-status', '--cached', '-z', 'HEAD',
+            stdout=subprocess.PIPE)
+
+        # if no changes exist, exit early
+        if not p.stdout:
+            commit.error('no staged changes exist')
+
+        data = deque(p.stdout.strip('\x00').split('\x00'))
+        changes = defaultdict(OrderedSet)
+        while data:
+            status = data.popleft()
+            old_path = None
+            if status.startswith('R'):
+                status = 'R'
+                old_path = data.popleft()
+            path = data.popleft()
+            path_components = path.split(os.sep)
+            if path_components[0] in self._repo.categories and len(path_components) > 2:
+                if mo := self._ebuild_re.match(path):
+                    # ebuild changes
+                    try:
+                        atom = atom_cls(f"={mo.group('category')}/{mo.group('package')}")
+                        old = None
+                        if status == 'R' and (om := self._ebuild_re.match(old_path)):
+                            old = atom_cls(f"={om.group('category')}/{om.group('package')}")
+                        changes[PkgChange].add(PkgChange(
+                            self._repo.location, status, path, atom=atom, ebuild=True, old=old))
+                    except MalformedAtom:
+                        continue
+                else:
+                    # non-ebuild package level changes
+                    atom = atom_cls(os.sep.join(path_components[:2]))
+                    changes[PkgChange].add(
+                        PkgChange(self._repo.location, status, path, atom=atom, ebuild=False))
+            elif mo := self._eclass_re.match(path):
+                changes[EclassChange].add(
+                    EclassChange(self._repo.location, status, path, name=mo.group('name')))
+            else:
+                changes[path_components[0]].add(Change(self._repo.location, status, path))
+
+        return changes
+
+    @jit_attr
+    def all(self):
+        """Ordered set of all change objects."""
+        return OrderedFrozenSet(chain.from_iterable(self.data.values()))
 
     @jit_attr
     def pkg_changes(self):
@@ -400,11 +521,6 @@ class GitChanges(UserDict):
         """Ordered set of all ebuild change objects."""
         return OrderedFrozenSet(x for x in self.pkg_changes if x.ebuild)
 
-    @jit_attr
-    def paths(self):
-        """Ordered set of all staged paths."""
-        return OrderedFrozenSet(x.path for x in chain.from_iterable(self.data.values()))
-
     @jit_attr
     def prefix(self):
         """Determine commit message prefix using GLEP 66 as a guide.
@@ -412,9 +528,17 @@ class GitChanges(UserDict):
         See https://www.gentoo.org/glep/glep-0066.html#commit-messages for
         details.
         """
-        # changes limited to a single type
-        if len(self.data) == 1:
-            change_type, change_objs = next(iter(self.data.items()))
+        single = len(self.data) == 1
+        pkg_with_profile_update = (
+            len(self.data) == 2 and set(self.data) == {PkgChange, 'profiles'})
+
+        if single or pkg_with_profile_update:
+            if single:
+                change_type, change_objs = next(iter(self.data.items()))
+            else:
+                change_type = PkgChange
+                change_objs = self.data[PkgChange]
+
             if len(change_objs) == 1:
                 # changes limited to a single object
                 return change_objs[0].prefix
@@ -443,19 +567,46 @@ class GitChanges(UserDict):
                     if self.pkg_changes[0].path.endswith('/Manifest'):
                         return 'update Manifest'
                     elif self.pkg_changes[0].path.endswith('/metadata.xml'):
-                        if summary := MetadataSummary(self._options, self.pkg_changes).generate():
-                            return summary
-            elif summary := PkgSummary(self._options, self.ebuild_changes).generate():
-                return summary
+                        return str(MetadataSummary(self._options, self.pkg_changes))
+            return str(PkgSummary(self._options, self.ebuild_changes))
         return ''
 
 
 @dataclass(frozen=True)
 class Change:
     """Generic file change."""
+    repo: str
     status: str
     path: str
 
+    @property
+    def full_path(self):
+        return pjoin(self.repo, self.path)
+
+    def read(self):
+        """Read data from the change's file."""
+        try:
+            with open(self.full_path, 'r', encoding='utf-8') as f:
+                data = f.read()
+        except (FileNotFoundError, UnicodeDecodeError):
+            data = None
+        object.__setattr__(self, 'data', data)
+        return data
+
+    def update(self, data):
+        """Update the change's cached file data."""
+        if data != getattr(self, 'data', None):
+            object.__setattr__(self, 'data', data)
+        return self
+
+    def sync(self):
+        """Write the change's cached file data back to its file."""
+        try:
+            with open(self.full_path, 'w', encoding='utf-8') as f:
+                f.write(self.data)
+        except AttributeError:
+            pass
+
     @property
     def prefix(self):
         if os.sep in self.path:
@@ -487,59 +638,6 @@ class PkgChange(Change):
         return f'{self.atom.key}: '
 
 
-def determine_changes(options):
-    """Determine changes staged in git."""
-    # stage changes as requested
-    if options.git_add_arg:
-        git.run('add', options.git_add_arg, options.cwd)
-
-    # determine staged changes
-    p = git.run(
-        'diff', '--name-status', '--cached', '-z',
-        stdout=subprocess.PIPE)
-
-    # ebuild path regex, validation is handled on instantiation
-    _ebuild_re = re.compile(r'^(?P<category>[^/]+)/[^/]+/(?P<package>[^/]+)\.ebuild$')
-    _eclass_re = re.compile(r'^eclass/(?P<name>[^/]+\.eclass)$')
-
-    # if no changes exist, exit early
-    if not p.stdout:
-        commit.error('no staged changes exist')
-
-    data = deque(p.stdout.strip('\x00').split('\x00'))
-    changes = defaultdict(OrderedSet)
-    while data:
-        status = data.popleft()
-        old_path = None
-        if status.startswith('R'):
-            status = 'R'
-            old_path = data.popleft()
-        path = data.popleft()
-        path_components = path.split(os.sep)
-        if path_components[0] in options.repo.categories and len(path_components) > 2:
-            if mo := _ebuild_re.match(path):
-                # ebuild changes
-                try:
-                    atom = atom_cls(f"={mo.group('category')}/{mo.group('package')}")
-                    old = None
-                    if status == 'R' and (om := _ebuild_re.match(old_path)):
-                        old = atom_cls(f"={om.group('category')}/{om.group('package')}")
-                    changes[PkgChange].add(PkgChange(
-                        status, path, atom=atom, ebuild=True, old=old))
-                except MalformedAtom:
-                    continue
-            else:
-                # non-ebuild package level changes
-                atom = atom_cls(os.sep.join(path_components[:2]))
-                changes[PkgChange].add(PkgChange(status, path, atom=atom, ebuild=False))
-        elif mo := _eclass_re.match(path):
-            changes[EclassChange].add(EclassChange(status, path, name=mo.group('name')))
-        else:
-            changes[path_components[0]].add(Change(status, path))
-
-    return GitChanges(options, changes)
-
-
 def determine_msg_args(options, changes):
     """Determine message-related arguments used with `git commit`."""
     args = []
@@ -567,32 +665,34 @@ def determine_msg_args(options, changes):
         elif changes.prefix:
             # use generated summary if a generated prefix exists
             message.append(changes.prefix + changes.summary)
+        else:
+            # use empty string to force user input
+            message.append('')
 
-        if message or options.footer:
-            tmp = tempfile.NamedTemporaryFile(mode='w')
-            tmp.write(message[0])
-            if len(message) > 1:
-                # wrap body paragraphs at 85 chars
-                body = ('\n'.join(textwrap.wrap(x, width=85)) for x in message[1:])
-                tmp.write('\n\n' + '\n\n'.join(body))
+        tmp = tempfile.NamedTemporaryFile(mode='w')
+        tmp.write(message[0])
+        if len(message) > 1:
+            # wrap body paragraphs at 85 chars
+            body = ('\n'.join(textwrap.wrap(x, width=85)) for x in message[1:])
+            tmp.write('\n\n' + '\n\n'.join(body))
 
-            # add footer tags
-            if options.footer:
-                tmp.write('\n\n')
-                for tag, url in options.footer:
-                    tmp.write(f'{tag}: {url}\n')
+        # add footer tags
+        if options.footer:
+            tmp.write('\n\n')
+            for tag, value in options.footer:
+                tmp.write(f'{tag}: {value}\n')
 
-            tmp.flush()
+        tmp.flush()
 
-            # force `git commit` to open an editor for uncompleted summary
-            if not message[0] or message[0].endswith(' '):
-                args.extend(['-t', tmp.name])
-            else:
-                args.extend(['-F', tmp.name])
+        # force `git commit` to open an editor for uncompleted summary
+        if not message[0] or message[0].endswith(' '):
+            args.extend(['-t', tmp.name])
+        else:
+            args.extend(['-F', tmp.name])
 
-            # Explicitly register tempfile removal so the object isn't garbage
-            # collected and removed when leaving function scope.
-            atexit.register(tmp.close)
+        # Explicitly register tempfile removal so the object isn't garbage
+        # collected and removed when leaving function scope.
+        atexit.register(tmp.close)
 
     return args
 
@@ -617,14 +717,11 @@ def _commit_validate(parser, namespace):
         namespace.commit_args.extend(['--signoff', '--gpg-sign'])
 
 
-@commit.bind_main_func
-def _commit(options, out, err):
+def update_manifests(options, out, err, changes):
+    """Update package manifests for any staged ebuild changes."""
     repo = options.repo
-    git_add_files = []
-    # determine changes from staged files
-    changes = determine_changes(options)
+    untracked_ebuild_re = re.compile(r'^\?\? (?P<category>[^/]+)/[^/]+/(?P<package>[^/]+)\.ebuild$')
 
-    _untracked_ebuild_re = re.compile(r'^\?\? (?P<category>[^/]+)/[^/]+/(?P<package>[^/]+)\.ebuild$')
     # update manifests for existing packages
     if atoms := {x.atom.unversioned_atom for x in changes.ebuild_changes}:
         if pkgs := {x.versioned_atom for x in repo.itermatch(packages.OrRestriction(*atoms))}:
@@ -633,7 +730,7 @@ def _commit(options, out, err):
                 'status', '--porcelain=v1', '-u', '-z', "*.ebuild",
                 cwd=repo.location, stdout=subprocess.PIPE)
             for path in p.stdout.strip('\x00').split('\x00'):
-                if mo := _untracked_ebuild_re.match(path):
+                if mo := untracked_ebuild_re.match(path):
                     try:
                         untracked = atom_cls(f"={mo.group('category')}/{mo.group('package')}")
                         pkgs.discard(untracked)
@@ -648,20 +745,34 @@ def _commit(options, out, err):
                 return 1
 
             # include existing Manifest files for staging
-            manifests = (pjoin(repo.location, f'{x.key}/Manifest') for x in atoms)
-            git_add_files.extend(filter(os.path.exists, manifests))
+            manifests = (pjoin(repo.location, f'{x.key}/Manifest') for x in pkgs)
+            options.git_add_files.extend(filter(os.path.exists, manifests))
+
+    return 0
+
+
+@commit.bind_main_func
+def _commit(options, out, err):
+    repo = options.repo
+
+    # determine changes from staged files
+    changes = GitChanges(options)
+
+    # update package manifests
+    if update_manifests(options, out, err, changes):
+        return 1
 
     # mangle files
     if options.mangle:
         # don't mangle FILESDIR content
         skip_regex = re.compile(rf'^{repo.location}/[^/]+/[^/]+/files/.+$')
         mangler = GentooMangler if options.gentoo_repo else Mangler
-        paths = (pjoin(repo.location, x) for x in changes.paths)
-        git_add_files.extend(mangler(paths, skip_regex=skip_regex))
+        files = mangler(changes.all, skip_regex=skip_regex)
+        options.git_add_files.extend(files)
 
     # stage modified files
-    if git_add_files:
-        git.run('add', *git_add_files, cwd=repo.location)
+    if options.git_add_files:
+        git.run('add', *options.git_add_files, cwd=repo.location)
 
     # scan staged changes for QA issues if requested
     if options.scan:
