@@ -170,7 +170,7 @@ class TestPkgdevCommit:
         self.args = ['pkgdev', 'commit'] + self.scan_args
 
     def test_empty_repo(self, capsys, repo, make_git_repo):
-        git_repo = make_git_repo(repo.location)
+        git_repo = make_git_repo(repo.location, commit=True)
         with patch('sys.argv', self.args), \
                 pytest.raises(SystemExit) as excinfo, \
                 chdir(git_repo.path):
@@ -182,7 +182,7 @@ class TestPkgdevCommit:
 
     def test_git_message_opts(self, repo, make_git_repo, tmp_path):
         """Verify message-related options are passed through to `git commit`."""
-        git_repo = make_git_repo(repo.location)
+        git_repo = make_git_repo(repo.location, commit=True)
         repo.create_ebuild('cat/pkg-0')
         git_repo.add_all('cat/pkg-0', commit=False)
         path = str(tmp_path / 'msg')
@@ -509,9 +509,208 @@ class TestPkgdevCommit:
             os.remove(pjoin(git_repo.path, f'cat/pkg/pkg-{v}.ebuild'))
         assert commit() == 'cat/pkg: drop versions'
 
-        # treeclean
+        # package rename
+        shutil.copytree(pjoin(git_repo.path, 'cat/pkg'), pjoin(git_repo.path, 'newcat/pkg'))
         shutil.rmtree(pjoin(git_repo.path, 'cat/pkg'))
-        assert commit() == 'cat/pkg: treeclean'
+        assert commit() == 'newcat/pkg: rename cat/pkg'
+
+        # treeclean
+        shutil.rmtree(pjoin(git_repo.path, 'newcat/pkg'))
+        assert commit() == 'newcat/pkg: treeclean'
+
+    def test_metadata_summaries(self, capsys, repo, make_git_repo):
+        git_repo = make_git_repo(repo.location)
+        pkgdir = os.path.dirname(repo.create_ebuild('cat/pkg-0'))
+        # stub metadata
+        with open(pjoin(pkgdir, 'metadata.xml'), 'w') as f:
+            f.write(textwrap.dedent("""\
+                <?xml version="1.0" encoding="UTF-8"?>
+                <!DOCTYPE pkgmetadata SYSTEM "http://www.gentoo.org/dtd/metadata.dtd">
+                <pkgmetadata>
+                    <maintainer type="person">
+                        <email>person@email.com</email>
+                        <name>Person</name>
+                    </maintainer>
+                </pkgmetadata>
+            """))
+        git_repo.add_all('cat/pkg-0')
+
+        def commit():
+            with os_environ(GIT_EDITOR="sed -i '1s/$/summary/'"), \
+                    patch('sys.argv', self.args + ['-a']), \
+                    pytest.raises(SystemExit) as excinfo, \
+                    chdir(git_repo.path):
+                self.script()
+            assert excinfo.value.code == 0
+            out, err = capsys.readouterr()
+            assert err == out == ''
+            message = git_repo.log(['-1', '--pretty=tformat:%B', 'HEAD'])
+            return message[0]
+
+        # add yourself
+        with open(pjoin(pkgdir, 'metadata.xml'), 'w') as f:
+            f.write(textwrap.dedent("""\
+                <?xml version="1.0" encoding="UTF-8"?>
+                <!DOCTYPE pkgmetadata SYSTEM "http://www.gentoo.org/dtd/metadata.dtd">
+                <pkgmetadata>
+                    <maintainer type="person">
+                        <email>person@email.com</email>
+                        <name>Person</name>
+                    </maintainer>
+                    <maintainer type="person">
+                        <email>first.last@email.com</email>
+                        <name>First Last</name>
+                    </maintainer>
+                </pkgmetadata>
+            """))
+        assert commit() == 'cat/pkg: add myself as a maintainer'
+
+        # drop yourself
+        with open(pjoin(pkgdir, 'metadata.xml'), 'w') as f:
+            f.write(textwrap.dedent("""\
+                <?xml version="1.0" encoding="UTF-8"?>
+                <!DOCTYPE pkgmetadata SYSTEM "http://www.gentoo.org/dtd/metadata.dtd">
+                <pkgmetadata>
+                    <maintainer type="person">
+                        <email>person@email.com</email>
+                        <name>Person</name>
+                    </maintainer>
+                </pkgmetadata>
+            """))
+        assert commit() == 'cat/pkg: drop myself as a maintainer'
+
+        # drop to maintainer-needed
+        with open(pjoin(pkgdir, 'metadata.xml'), 'w') as f:
+            f.write(textwrap.dedent("""\
+                <?xml version="1.0" encoding="UTF-8"?>
+                <!DOCTYPE pkgmetadata SYSTEM "http://www.gentoo.org/dtd/metadata.dtd">
+                <pkgmetadata>
+                </pkgmetadata>
+            """))
+        assert commit() == 'cat/pkg: drop to maintainer-needed'
+
+        # add random maintainer
+        with open(pjoin(pkgdir, 'metadata.xml'), 'w') as f:
+            f.write(textwrap.dedent("""\
+                <?xml version="1.0" encoding="UTF-8"?>
+                <!DOCTYPE pkgmetadata SYSTEM "http://www.gentoo.org/dtd/metadata.dtd">
+                <pkgmetadata>
+                    <maintainer type="person">
+                        <email>person@email.com</email>
+                        <name>Person</name>
+                    </maintainer>
+                </pkgmetadata>
+            """))
+        assert commit() == 'cat/pkg: update maintainers'
+
+        # add allarches tag
+        with open(pjoin(pkgdir, 'metadata.xml'), 'w') as f:
+            f.write(textwrap.dedent("""\
+                <?xml version="1.0" encoding="UTF-8"?>
+                <!DOCTYPE pkgmetadata SYSTEM "http://www.gentoo.org/dtd/metadata.dtd">
+                <pkgmetadata>
+                    <maintainer type="person">
+                        <email>person@email.com</email>
+                        <name>Person</name>
+                    </maintainer>
+                    <stabilize-allarches/>
+                </pkgmetadata>
+            """))
+        assert commit() == 'cat/pkg: mark ALLARCHES'
+
+        # drop allarches tag
+        with open(pjoin(pkgdir, 'metadata.xml'), 'w') as f:
+            f.write(textwrap.dedent("""\
+                <?xml version="1.0" encoding="UTF-8"?>
+                <!DOCTYPE pkgmetadata SYSTEM "http://www.gentoo.org/dtd/metadata.dtd">
+                <pkgmetadata>
+                    <maintainer type="person">
+                        <email>person@email.com</email>
+                        <name>Person</name>
+                    </maintainer>
+                </pkgmetadata>
+            """))
+        assert commit() == 'cat/pkg: drop ALLARCHES'
+
+        # add upstream metadata
+        with open(pjoin(pkgdir, 'metadata.xml'), 'w') as f:
+            f.write(textwrap.dedent("""\
+                <?xml version="1.0" encoding="UTF-8"?>
+                <!DOCTYPE pkgmetadata SYSTEM "http://www.gentoo.org/dtd/metadata.dtd">
+                <pkgmetadata>
+                    <maintainer type="person">
+                        <email>person@email.com</email>
+                        <name>Person</name>
+                    </maintainer>
+                    <upstream>
+                        <remote-id type="github">pkgcore/pkgdev</remote-id>
+                        <remote-id type="pypi">pkgdev</remote-id>
+                    </upstream>
+                </pkgmetadata>
+            """))
+        assert commit() == 'cat/pkg: add github, pypi upstream metadata'
+
+        # remove upstream metadata
+        with open(pjoin(pkgdir, 'metadata.xml'), 'w') as f:
+            f.write(textwrap.dedent("""\
+                <?xml version="1.0" encoding="UTF-8"?>
+                <!DOCTYPE pkgmetadata SYSTEM "http://www.gentoo.org/dtd/metadata.dtd">
+                <pkgmetadata>
+                    <maintainer type="person">
+                        <email>person@email.com</email>
+                        <name>Person</name>
+                    </maintainer>
+                    <upstream>
+                        <remote-id type="github">pkgcore/pkgdev</remote-id>
+                    </upstream>
+                </pkgmetadata>
+            """))
+        assert commit() == 'cat/pkg: remove pypi upstream metadata'
+
+        # update upstream metadata
+        with open(pjoin(pkgdir, 'metadata.xml'), 'w') as f:
+            f.write(textwrap.dedent("""\
+                <?xml version="1.0" encoding="UTF-8"?>
+                <!DOCTYPE pkgmetadata SYSTEM "http://www.gentoo.org/dtd/metadata.dtd">
+                <pkgmetadata>
+                    <maintainer type="person">
+                        <email>person@email.com</email>
+                        <name>Person</name>
+                    </maintainer>
+                    <upstream>
+                        <remote-id type="github">pkgcore/pkgcheck</remote-id>
+                    </upstream>
+                </pkgmetadata>
+            """))
+        assert commit() == 'cat/pkg: update upstream metadata'
+
+    def test_no_summary(self, capsys, repo, make_git_repo):
+        git_repo = make_git_repo(repo.location)
+        repo.create_ebuild('cat/pkg-0')
+        git_repo.add_all('cat/pkg-0')
+
+        def commit(args):
+            with os_environ(GIT_EDITOR="sed -i '1s/$/summary/'"), \
+                    patch('sys.argv', self.args + args), \
+                    pytest.raises(SystemExit) as excinfo, \
+                    chdir(git_repo.path):
+                self.script()
+            assert excinfo.value.code == 0
+            out, err = capsys.readouterr()
+            assert err == out == ''
+            return git_repo.log(['-1', '--pretty=tformat:%B', 'HEAD'])
+
+        # no commit message content
+        for i in range(10):
+            with open(pjoin(git_repo.path, f'file-a-{i}'), 'w') as f:
+                f.write('stub\n')
+        assert commit(['-a']) == ['summary']
+
+        # footer exists with no generated summary
+        for i in range(10):
+            with open(pjoin(git_repo.path, f'file-b-{i}'), 'w') as f:
+                f.write('stub\n')
+        assert commit(['-a', '-T', 'tag:value']) == ['summary', '', 'Tag: value']
 
     def test_non_gentoo_file_mangling(self, repo, make_git_repo):
         git_repo = make_git_repo(repo.location)
